% This is a demonstration of GIH
addpath common_GIH
clear;


%------------------------------------------------------------------------------
% prepare image
im1	= imread('rd1.bmp');
if size(im1,3)>1			im1	= rgb2gray(im1); 	end
im1	= double(im1)/255;


%-------------------------------------------------------------------------------
% GIH computating
disp('Demonstrate computation of GIH');
disp('   Left click on the image to get an interest point to start a computing.');
disp('   Right click to end.');
alpha		= .99;				% aspect ratio, in (0..1)
n_color		= 11;				% number of color (intensity) bin 
n_geo		= 8;				% number of geodesic distance bin
geo_range	= 1.6;				% geodesic range of the support region
geo_reso	= geo_range/n_geo/8;	% geodesic distance gap
msk1		= ones(size(im1));	% foreground mask
A			= [.95, 1, 1.05];	% lighting coeffecients, affine lighting model I=aI+b
ifig		= -1;				% if >0, demonstrate computation

idemo	= 1;
figure(idemo);	clf;	colormap(gray);
subplot(1,3,1); 
imagesc(im1);

[nR,nC]		= size(im1);
[x,y,btn]	= ginput(1);
while btn==1
	if 1<x & x<nC & 1<y & y<nR
	
		% compute GIH
		t	= clock;
		[GHs, T] = ...
			GIH(	im1, ...		% input intensity image, value in [0..1]
					x, ...			% x of the interest point
					y, ...			% y of the interest point
					alpha, ...		% aspect ratio, in (0..1)
					n_color, ...	% number of color (intensity) bin 
					n_geo, ...		% number of geodesic distance bin
					geo_range, ...	% geodesic range of the support region
					geo_reso, ...	% sampling gap (used by contour extraction)
					msk1, ... 		% foreground mask
					A, ...			% lighting coeffecients, affine lighting model I=aI+b
					ifig ...		% if >0, demonstrate computation
				);
						
% 		% the simplest calling is						
% 		[GHs, T]	= GIH(	im1, ...		% input intensity image, value in [0..1]
% 							x, ...			% x of the interest point
% 							y ...			% y of the interest point
% 						);
					
		t	= etime(clock,t);
						
		%- demostrate
		figure(idemo);	clf;	colormap(gray); hold on;
		subplot(1,3,1); 
		imagesc(im1);	hold on;
		maxT	= max(T(:));
		
		% draw the support region
		CT		= contourc(T,[maxT-.1 maxT-.1]);			% get geodesic level curves
		n_CT	= size(CT,2);
		iHead		= 1;
		while iHead<n_CT
			n_cont	= CT(2,iHead);
			C_tmp	= CT(:, iHead+1:iHead+n_cont);		% the contour to be dealing
			iHead	= iHead+n_cont+1;					% pointer to the next contour
			plot(C_tmp(1,:), C_tmp(2,:),'r-');
		end

		plot(x,y,'rx');
		title(['left click: choose interest point; right click: over']);
		xlabel('boundary of support region is shown in the red contour');
		
	
		% show the geodesic distance
		subplot(1,3,2); 
		imagesc(T);	hold on;
		plot(x,y,'rx');
		title(['geodesic dist.']);
		xlabel(['total time for GIH = ' num2str(t) 'sec.']);

		n_A	= length(A);
		for ia=1:n_A
			a	= A(ia);
			H	= reshape(GHs(:,ia), n_color, n_geo);
			
			c	= ceil(n_A/2);
			subplot(c, 6, ceil(ia/2)*4+ia);
			imagesc(H);
			title(['a=' num2str(a)]);
			if ia==n_A
				xlabel('geodesic');
				ylabel('intensity');
			end
		end
	end
	
	[x,y,btn]	= ginput(1);
end

			
%-------------------------------------------------------------------------------
% GIH distance
im2	= imread('rd2.bmp');
if size(im2,3)>1			im2	= rgb2gray(im2); 	end
im2	= double(im2)/255;

disp('Distance between GIHs at two points');
msk2		= ones(size(im2));	% foreground mask
ifig		= -1;				% if >0, demonstrate computation

[nR1,nC1]	= size(im1);
[nR2,nC2]	= size(im2);

bEnd	= 0;
idemo	= idemo+1;
while ~bEnd
	figure(idemo);	clf;	colormap(gray); hold on;
	
	% get the first interest point
	subplot(2,3,2); 
	imagesc(im2);	
	subplot(2,3,1); 
	imagesc(im1);	hold on;
	title('click on this image for interest point');
	[x1,y1,btn]	= ginput(1);
	while 1>x1 | x1>nC1 | 1>y1 | y1>nR1
		[x1,y1,btn]	= ginput(1);
	end
	plot(x1,y1,'rx');
	title('interest point 1 done.');
	
	% get the second interest point
	subplot(2,3,2); 
	imagesc(im2);	hold on;
	title('click on this image for interest point');
	[x2,y2,btn]	= ginput(1);
	while 1>x2 | x2>nC2 | 1>y2 | y2>nR2
		[x2,y2,btn]	= ginput(1);
	end
	plot(x2,y2,'rx');
	title('interest point 2 done.');
	drawnow;

	% compute GIHs and the distance between them
	t	= clock;
	[GHs1, T]	= GIH(	im1, ...		% input intensity image, value in [0..1]
						x1, ...			% x of the interest point
						y1, ...			% y of the interest point
						alpha, ...		% aspect ratio, in (0..1)
						n_color, ...	% number of color (intensity) bin 
						n_geo, ...		% number of geodesic distance bin
						geo_range, ...	% geodesic range of the support region
						geo_reso, ...	% sampling gap (used by contour extraction)
						msk1, ... 		% foreground mask
						A, ...			% lighting coeffecients, affine lighting model I=aI+b
						ifig ...		% if >0, demonstrate computation
					);
	
	% show the geodesic distance
	t	= etime(clock,t);
	subplot(2,3,4); 
	imagesc(T);	hold on;
	plot(x1,y1,'rx');
	title(['geodesic dist.']);	xlabel(['total time for GIH = ' num2str(t) 'sec.']);
	
	t	= clock;
	[GHs2, T]	= GIH(	im2, ...		% input intensity image, value in [0..1]
						x2, ...			% x of the interest point
						y2, ...			% y of the interest point
						alpha, ...		% aspect ratio, in (0..1)
						n_color, ...	% number of color (intensity) bin 
						n_geo, ...		% number of geodesic distance bin
						geo_range, ...	% geodesic range of the support region
						geo_reso, ...	% sampling gap (used by contour extraction)
						msk2, ... 		% foreground mask
						A, ...			% lighting coeffecients, affine lighting model I=aI+b
						ifig ...		% if >0, demonstrate computation
					);
					
	% show the geodesic distance
	t	= etime(clock,t);
	subplot(2,3,5); 
	imagesc(T);	hold on;
	plot(x2,y2,'rx');
	title(['geodesic dist.']);	xlabel(['total time for GIH = ' num2str(t) 'sec.']);

	dis	= distmat_GIH(GHs1,GHs2);
	
	%- demostrate
	n_A	= length(A);
	for ia=1:n_A
		a	= A(ia);
		H1	= reshape(GHs1(:,ia), n_color, n_geo);
		H2	= reshape(GHs2(:,ia), n_color, n_geo);

		subplot(n_A, 6, (ia-1)*6+5);
		imagesc(H1);	axis off;
		title(['a=' num2str(a)]);
		subplot(n_A, 6, (ia-1)*6+6);
		imagesc(H2);	axis off;
		
		if ia==1
			title(['dist = ' num2str(dis)]);
		end
	end

	%-
	title('LEFT click to continue, RIGHT to end');
	disp('LEFT click to continue, RIGHT to end');
	[x,y,btn]	= ginput(1);
	bEnd	= btn~=1;
end
