% function C = distmat_GIH( Hs1, Hs2 )
% 	Compute the distance matrix between distances of GIHs at two set of points
%		from two images.
%
%	Input:
%	Hs1, Hs2 : GIHs at two point sets (usually from two images)
%				Both are 3D matrix, in the format of (n_color*n_geo) X n_A X n_pt
%				where n_A is the number of lighting coefficients used to dealing with
%				affine lighting change, n_pt is the number of interest points.
%	Output:
%		C	: the distance matrix, let Hs1 contains n1 point and Hs2 contains n2 points,
%				then C is n1 x n2 matrix, such that C(i,j) is the distance between the 
%				GIH at the i-th point in Hs1 and j-th point at Hs2.
%
function C = distmat_GIH( Hs1, Hs2 )

[n_bin,n_A,n1]	= size(Hs1);
[n_bin,n_A,n2]	= size(Hs2);

costmats	= zeros(n1,n2,2*n_A-1);
for ia=1:n_A	
	costmats(:,:,ia)	  = chi2_dist_C(Hs1(:,1,:),Hs2(:,ia,:));
end
for ia=2:n_A	
	costmats(:,:,n_A+ia-1)= chi2_dist_C(Hs1(:,ia,:),Hs2(:,1,:));
end
C	= min(costmats,[],3);
