% sampling along a curve according to the geodesic distance

function [XIs,YIs,ZIs] = geo_sampling(C,d_len,im,alpha)

	%- prepare the points
	[nR,nC]	= size(im);
	if size(C,2)>size(C,1)	C=C';	end
	Xs		= C(:,1);
	Ys		= C(:,2);
	iXs		= min(max(round(Xs),1),nC);
	iYs		= min(max(round(Ys),1),nR);
	idx		= (iXs-1)*nR+iYs;
	Zs		= double(im(idx));
	alpha2	= alpha*alpha;
		
	%- prepare the distances
	difXs		= diff(Xs);
	difYs		= diff(Ys);
	difZs		= diff(Zs);
	seg_lens	= [0; sqrt(difXs.^2+difYs.^2+alpha2*difZs.^2)];
	n_pt		= length(Xs);
	for(ii=2:n_pt)
		seg_lens(ii)	= seg_lens(ii)+seg_lens(ii-1);
	end
	len_all		= seg_lens(end);

	%- prepare the outputs
	n_samp		= floor(len_all/d_len)+1;
	XIs			= zeros(n_samp,1);
	YIs			= XIs;
	ZIs			= XIs;

	%- interpolate to get each point
	XIs(1)	= Xs(1);
	YIs(1)	= Ys(1);
	ZIs(1)	= Zs(1);
	n_count	= 1;
	cur_len	= d_len;
	for(ii=2:n_pt)
		while cur_len <= seg_lens(ii)  & n_count<n_samp
			% interpolate a point
			r	= (cur_len-seg_lens(ii-1)) / (seg_lens(ii)-seg_lens(ii-1));
			x	= Xs(ii-1)+r*(Xs(ii)-Xs(ii-1));
			y	= Ys(ii-1)+r*(Ys(ii)-Ys(ii-1));
			z	= Zs(ii-1)+r*(Zs(ii)-Zs(ii-1));
			n_count	= n_count+1;
			XIs(n_count)	= x;
			YIs(n_count)	= y;
			ZIs(n_count)	= z;
			cur_len	= cur_len+d_len;
		end
	end
	
	if n_count<n_samp
		XIs	= XIs(1:n_count);
		YIs	= YIs(1:n_count);
		ZIs	= ZIs(1:n_count);
		n_samp = n_count;
	end
		
	% test
	if 0
		figure(97);	clf; hold on;
		imagesc(im);	colormap(gray);
		plot(Xs,Ys,'r.-');
		plot(XIs,YIs,'b+');
		title(['n_samp=' i2s(n_samp) ', len=' i2s(length(XIs))]);
		keyboard;
	end
